@@ -1,11 +1,6 @@
 .onAttach <- function(...) {
   ver <- utils::packageVersion("projpred")
   msg <- paste0("This is projpred version ", ver, ".")
-  msg <- paste0(msg, " ", "NOTE: In projpred 2.7.0, the default search method ",
-                "was set to \"forward\" (for all kinds of models). ",
-                "The corresponding NOTE thrown by varsel() and cv_varsel() ",
-                "can be suppressed by setting ",
-                "`options(projpred.mssg_method_changed = FALSE)`.")
   packageStartupMessage(msg)
 }
 
@@ -17,7 +12,10 @@ nms_y_wobs_test <- function(wobs_nm = "wobs") {
   c("y", "y_oscale", wobs_nm)
 }
 
-weighted.sd <- function(x, w, na.rm = FALSE) {
+.weighted_sd <- function(x, w, na.rm = FALSE) {
+  if (length(w) == 1 && length(x) > 1) {
+    w <- rep_len(w, length.out = length(x))
+  }
   if (na.rm) {
     ind <- !is.na(w) & !is.na(x)
     n <- sum(ind)
@@ -25,6 +23,9 @@ weighted.sd <- function(x, w, na.rm = FALSE) {
     n <- length(x)
     ind <- rep(TRUE, n)
   }
+  if (n %in% c(0, 1)) {
+    return(NA_real_)
+  }
   w <- w / sum(w[ind])
   m <- sum(x[ind] * w[ind])
   sqrt(n / (n - 1) * sum(w[ind] * (x[ind] - m)^2))
@@ -63,17 +64,25 @@ ilinkfun_raw <- function(x, link_nm) {
   return(basic_ilink(x))
 }
 
-auc <- function(x) {
+.auc <- function(x) {
   resp <- x[, 1]
   pred <- x[, 2]
-  wcv <- x[, 3]
-  n <- nrow(x)
-  ord <- order(pred, decreasing = TRUE)
+  wobs <- x[, 3]
+
+  # Make it explicit that `x` should not be used anymore:
+  rm(x)
+
+  ord <- order(pred, decreasing = TRUE, na.last = FALSE)
+  n <- length(ord)
+
   resp <- resp[ord]
   pred <- pred[ord]
-  wcv <- wcv[ord]
-  w0 <- w1 <- wcv
-  stopifnot(all(resp %in% c(0, 1)))
+  wobs <- wobs[ord]
+
+  w0 <- w1 <- wobs
+  stopifnot(all(na.omit(resp) %in% c(0, 1)))
+  w0[is.na(resp)] <- NA # ensure that `NA`s in `resp` propagate to the output
+  w1[is.na(resp)] <- NA # ensure that `NA`s in `resp` propagate to the output
   w0[resp == 1] <- 0 # for calculating the false positive rate (fpr)
   w1[resp == 0] <- 0 # for calculating the true positive rate (tpr)
   cum_w0 <- cumsum(w0)
@@ -111,7 +120,7 @@ bootstrap <- function(x, fun = mean, B = 2000, seed = NA, ...) {
   bsstat <- rep(NA, B)
   for (i in 1:B) {
     bsind <- sample(seq_x, replace = TRUE)
-    bsstat[i] <- fun(if (is_vector) x[bsind] else x[bsind, ], ...)
+    bsstat[i] <- fun(if (is_vector) x[bsind] else x[bsind, , drop = FALSE], ...)
   }
   return(bsstat)
 }
@@ -144,9 +153,10 @@ validate_vsel_object_stats <- function(object, stats, resp_oscale = TRUE) {
   }
   resp_oscale <- object$refmodel$family$for_latent && resp_oscale
 
-  trad_stats <- c("elpd", "mlpd", "mse", "rmse", "acc", "pctcorr", "auc")
+  trad_stats <- c("elpd", "mlpd", "gmpd", "mse", "rmse", "R2",
+                  "acc", "pctcorr", "auc")
   trad_stats_binom_only <- c("acc", "pctcorr", "auc")
-  augdat_stats <- c("elpd", "mlpd", "acc", "pctcorr")
+  augdat_stats <- c("elpd", "mlpd", "gmpd", "acc", "pctcorr")
   resp_oscale_stats_fac <- augdat_stats
 
   if (is.null(stats)) {
@@ -187,16 +197,21 @@ validate_vsel_object_stats <- function(object, stats, resp_oscale = TRUE) {
   return(invisible(TRUE))
 }
 
-validate_baseline <- function(refmodel, baseline, deltas) {
+validate_baseline <- function(vsel_obj, baseline, deltas) {
   stopifnot(!is.null(baseline))
   if (!(baseline %in% c("ref", "best"))) {
     stop("Argument 'baseline' must be either 'ref' or 'best'.")
   }
-  if (baseline == "ref" && deltas == TRUE && inherits(refmodel, "datafit")) {
+  if (baseline == "ref" && (identical(deltas, "mixed") || isTRUE(deltas)) &&
+      inherits(vsel_obj$refmodel, "datafit")) {
     # no reference model (or the results missing for some other reason),
-    # so cannot compute differences between the reference model and submodels
-    stop("Cannot use deltas = TRUE and baseline = 'ref' when there is no ",
-         "reference model.")
+    # so cannot compute differences (or ratios) vs. the reference model
+    stop("Cannot use `deltas = TRUE` (or `deltas = \"mixed\"`) and ",
+         "`baseline = \"ref\"` when there is no reference model.")
+  }
+  if (baseline == "best" && vsel_obj$cv_method == "LOO" &&
+      isTRUE(vsel_obj$nloo < vsel_obj$refmodel$nobs)) {
+    stop("Cannot use `baseline = \"best\"` in case of subsampled LOO-CV.")
   }
   return(baseline)
 }
@@ -261,6 +276,7 @@ get_standard_y <- function(y, weights, fam) {
 #   * `mu`: An \eqn{N \times S_{\mathrm{prj}}}{N x S_prj} matrix of expected
 #   values for \eqn{y} (probabilities for the response categories in case of the
 #   augmented-data projection) for each draw/cluster.
+#   * `mu_offs`: Same as `mu`, but taking offsets into account.
 #   * `var`: An \eqn{N \times S_{\mathrm{prj}}}{N x S_prj} matrix of predictive
 #   variances for \eqn{y} for each draw/cluster which are needed for projecting
 #   the dispersion parameter (the predictive variances are NA for those families
@@ -270,6 +286,9 @@ get_standard_y <- function(y, weights, fam) {
 #   those families that do not have a dispersion parameter).
 #   * `wdraws_prj`: A vector of length \eqn{S_{\mathrm{prj}}}{S_prj} containing
 #   the weights for the projected draws/clusters.
+#   * `const_wdraws_prj`: A single logical value indicating whether all
+#   projected draws have the same weight.
+#   * `nprjdraws`: A single value: \eqn{S_{\mathrm{prj}}}{S_prj}.
 #   * `cl`: Cluster assignment for each posterior draw, that is, a vector that
 #   has length equal to the number of posterior draws and each value is an
 #   integer between 1 and \eqn{S_{\mathrm{prj}}}{S_prj}.
@@ -283,6 +302,9 @@ get_standard_y <- function(y, weights, fam) {
 #   `refmodel` object anyway). However, get_p_clust() intentionally seems to
 #   have been kept as general as possible and `wdraws_orig` is more general than
 #   `wdraws_ref`.
+#   * `clust_used`: A single logical value indicating whether clustering (i.e.,
+#   get_p_clust(), with the possibility that its output element `clust_used` is
+#   overwritten by its argument `clust_used_forced`) has been used.
 get_refdist <- function(refmodel, ndraws = NULL, nclusters = NULL,
                         thinning = TRUE,
                         throw_mssg_ndraws = getOption("projpred.mssg_ndraws",
@@ -311,6 +333,10 @@ get_refdist <- function(refmodel, ndraws = NULL, nclusters = NULL,
                            nclusters = nclusters)
     }
   } else {
+    if (length(unique(refmodel$wdraws_ref)) != 1) {
+      stop("Currently, projpred requires the reference model's posterior ",
+           "draws to have constant weights.")
+    }
     ndraws <- min(S, ndraws)
     if (ndraws <= 20 && throw_mssg_ndraws) {
       message("The number of draws to project is quite small (<= 20). In such ",
@@ -331,10 +357,12 @@ get_refdist <- function(refmodel, ndraws = NULL, nclusters = NULL,
       mu = refmodel$mu[, s_ind, drop = FALSE],
       mu_offs = refmodel$mu_offs[, s_ind, drop = FALSE],
       var = structure(predvar,
-                      nobs_orig = attr(refmodel$mu, "nobs_orig"),
+                      ndiscrete = attr(refmodel$mu, "ndiscrete"),
                       class = oldClass(refmodel$mu)),
       dis = refmodel$dis[s_ind],
       wdraws_prj = rep(1 / ndraws, ndraws),
+      const_wdraws_prj = TRUE,
+      nprjdraws = ndraws,
       cl = cl,
       wdraws_orig = rep(1, S),
       clust_used = FALSE
@@ -347,7 +375,8 @@ get_refdist <- function(refmodel, ndraws = NULL, nclusters = NULL,
 # Function for clustering the parameter draws:
 get_p_clust <- function(family, eta, mu, mu_offs, dis, nclusters = 10,
                         wobs = rep(1, dim(mu)[1]),
-                        wdraws = rep(1, dim(mu)[2]), cl = NULL) {
+                        wdraws = rep(1, dim(mu)[2]), cl = NULL,
+                        clust_used_forced = NULL) {
   # cluster the draws in the latent space if no clustering provided
   if (is.null(cl)) {
     # Note: A seed is not set here because this function is not exported and has
@@ -401,19 +430,21 @@ get_p_clust <- function(family, eta, mu, mu_offs, dis, nclusters = 10,
   # combine the results
   return(list(
     mu = structure(unname(centers),
-                   nobs_orig = attr(mu, "nobs_orig"),
+                   ndiscrete = attr(mu, "ndiscrete"),
                    class = oldClass(mu)),
     mu_offs = structure(unname(centers_offs),
-                        nobs_orig = attr(mu_offs, "nobs_orig"),
+                        ndiscrete = attr(mu_offs, "ndiscrete"),
                         class = oldClass(mu_offs)),
     var = structure(predvar,
-                    nobs_orig = attr(mu, "nobs_orig"),
+                    ndiscrete = attr(mu, "ndiscrete"),
                     class = oldClass(mu)),
     dis = dis_agg,
     wdraws_prj = wcluster,
+    const_wdraws_prj = length(unique(wcluster)) == 1,
+    nprjdraws = nclusters,
     cl = cl,
     wdraws_orig = wdraws,
-    clust_used = TRUE
+    clust_used = clust_used_forced %||% TRUE
   ))
 }
 
@@ -424,6 +455,22 @@ draws_subsample <- function(S, ndraws) {
   return(sample.int(S, size = ndraws))
 }
 
+# If no get_refdist() (or get_p_clust()) output is available to infer elements
+# `clust_used` (a single logical value indicating whether the given combination
+# of `ndraws` and `nclusters` will lead to clustered projected draws or not) and
+# `nprjdraws` (a single numeric value giving the number of (possibly clustered)
+# projected draws resulting from the given combination of `ndraws` and
+# `nclusters`), the following function can be used as a workaround (argument `S`
+# denotes the number of posterior draws in the reference model object):
+clust_info <- function(ndraws, nclusters, S) {
+  clust_used_out <- !is.null(nclusters) && nclusters < S
+  nprjdraws_out <- if (clust_used_out) nclusters else ndraws
+  return(list(
+    clust_used = clust_used_out,
+    nprjdraws = nprjdraws_out
+  ))
+}
+
 is_proj_list <- function(proj) {
   # Better use a formal class `proj_list`, but for now, use this workaround:
   is.list(proj) && length(proj) && all(sapply(proj, inherits, "projection"))
@@ -458,12 +505,10 @@ nlist <- function(...) {
   x
 }
 
-#' Execute a function call (deprecated)
+#' Execute a function call
 #'
 #' Execute a function call similar to [do.call()], but without deparsing
-#' function arguments. This function is deprecated and will be removed in a
-#' future release. Where possible, please use direct function calls instead. If
-#' this is not possible, please use [do.call()] instead.
+#' function arguments.
 #'
 #' @param what Either a function or a non-empty character string naming the
 #'   function to be called.
@@ -477,9 +522,6 @@ nlist <- function(...) {
 #' @keywords internal
 #' @export
 do_call <- function(what, args, pkg = NULL) {
-  warning("projpred::do_call() is deprecated and will be removed in a future ",
-          "release. Where possible, please use direct function calls instead. ",
-          "If this is not possible, please use base::do.call() instead.")
   call <- ""
   if (length(args)) {
     if (!is.list(args)) {
@@ -586,13 +628,61 @@ rbind2list <- function(x) {
   return(binded_list)
 }
 
-# Print out text via cat() if `verbose = TRUE`:
+cat_cls <- function(x) {
+  cls <- paste0("`", class(x), "`")
+  cat("Object of class", if (length(cls) > 1) "es" else "", " ",
+      paste(cls, collapse = ", "), "\n\n", sep = "")
+}
+
+# Print out text via message() if `verbose = TRUE`:
 verb_out <- function(..., verbose = TRUE) {
   if (verbose) {
-    cat(..., "\n", sep = "")
+    message(...)
   }
 }
 
+# Helper function intended for use in verbose messages:
+txt_clust_draws <- function(clust_used, nprjdraws) {
+  out <- paste0(nprjdraws, " ")
+  if (clust_used) {
+    out <- paste0(out, "cluster")
+  } else {
+    out <- paste0(out, "draw")
+  }
+  if (nprjdraws > 1) {
+    out <- paste0(out, "s")
+  }
+  return(out)
+}
+
+# Ensure that stderr() is used for throwing an error, even while sink()-ing or
+# capture.output()-ing with `type = "message"`:
+throw_err <- function(e) {
+  sink(type = "message")
+  stop(e)
+}
+
+# A wrapper for capture.output() with `type = "message"`, but throwing error
+# messages appropriately (i.e., only messages and warnings are captured).
+# Note: This function should only be used to filter out messages or warnings
+# (not to make downstream code dependent on catched messages or warnings), see
+# <https://github.com/stan-dev/loo/issues/227#issuecomment-1663499985>.
+capt_mssgs_warns <- function(expr) {
+  if (getOption("warn") == 0) {
+    warn_orig <- options(warn = 1)
+    on.exit(options(warn_orig))
+  }
+  mssgs_warns_capt <- utils::capture.output(tryCatch(expr, error = throw_err),
+                                            type = "message")
+  if (identical(Sys.getenv("RSTUDIO"), "1")) {
+    # Some RStudio versions (observed in RStudio 2025.05.0+496) add special
+    # characters/symbols, so remove them:
+    mssgs_warns_capt <- gsub("\033[GHgh]{1}2?;?", "", mssgs_warns_capt)
+  }
+  mssgs_warns_capt <- setdiff(mssgs_warns_capt, "")
+  return(mssgs_warns_capt)
+}
+
 # Parse the argument containing the observation weights (`wobs` or `weights`)
 # for the <family_object>$ppd() functions used by proj_predict():
 parse_wobs_ppd <- function(wobs, n_obs) {
@@ -637,7 +727,7 @@ reorder_ia <- function(ia, y) {
   if (length(ia_reordered) == 0) {
     ia_reordered <- NA_character_
   } else if (length(ia_reordered) > 1) {
-    stop("Unexpected length of `ia_reordered`. Please contact the package ",
+    stop("Unexpected length of `ia_reordered`. Please notify the package ",
          "maintainer.")
   }
   return(ia_reordered)
@@ -667,3 +757,83 @@ element_unq <- function(list_obj, nm) {
   }
   return(el_unq)
 }
+
+use_progressr <- function() {
+  getOption("projpred.use_progressr",
+            requireNamespace("progressr", quietly = TRUE) &&
+              interactive() &&
+              identical(foreach::getDoParName(), "doFuture"))
+}
+
+sqrt_cut0 <- function(x) {
+  if (!is.na(x) && sign(x) == -1) {
+    if (abs(x) < 1e3 * sqrt(.Machine$double.eps)) {
+      x <- 0
+    } else {
+      stop("Negative (and numerically non-zero) value used as input to ",
+           "sqrt_cut0().")
+    }
+  }
+  return(sqrt(x))
+}
+
+verbose_from_deprecated_options <- function(verbose, with_cv = FALSE,
+                                            proj_only = FALSE) {
+  if (!is.logical(verbose) || !is.null(getOption("projpred.verbose"))) {
+    return(verbose)
+  }
+  if (!is.null(getOption("projpred.extra_verbose")) &&
+      !is.null(getOption("projpred.verbose_project"))) {
+    warning(
+      "Global options `projpred.extra_verbose` and `projpred.verbose_project` ",
+      "are deprecated. Please use argument ",
+      "`verbose` instead. This argument can also be controlled via a ",
+      "corresponding global option. Now trying to find an appropriate value ",
+      "for argument `verbose`."
+    )
+    verbose <- as.logical(verbose) * (
+      proj_only * 2L +
+        (!proj_only) * (
+          (getOption("projpred.extra_verbose") ||
+             getOption("projpred.verbose_project")) *
+            (1L + with_cv + getOption("projpred.verbose_project")) +
+            1L
+        )
+    )
+  } else {
+    verbose <- verbose_from_deprecated_option(
+      verbose,
+      with_cv = with_cv,
+      proj_only = proj_only,
+      nm_option = "projpred.extra_verbose"
+    )
+    verbose <- verbose_from_deprecated_option(
+      verbose,
+      with_cv = with_cv,
+      proj_only = proj_only,
+      nm_option = "projpred.verbose_project"
+    )
+  }
+  return(verbose)
+}
+
+verbose_from_deprecated_option <- function(verbose, with_cv = FALSE,
+                                           proj_only = FALSE, nm_option) {
+  if (!is.null(getOption(nm_option))) {
+    warning(
+      "Global option `", nm_option, "` is deprecated. Please use argument ",
+      "`verbose` instead. This argument can also be controlled via a ",
+      "corresponding global option. Now trying to find an appropriate value ",
+      "for argument `verbose`."
+    )
+    verbose <- as.logical(verbose) * (
+      proj_only * 2L +
+        (!proj_only) * (
+          getOption(nm_option) *
+            (1L + with_cv + identical(nm_option, "projpred.verbose_project")) +
+            1L
+        )
+    )
+  }
+  return(verbose)
+}
