@@ -3,6 +3,12 @@ context("proj_predfun")
 test_that("repair_re() works for GLMMs", {
   # Setup -------------------------------------------------------------------
 
+  # In the past, there have been incompatibility issues between packages
+  # 'Matrix' and 'lme4', see commits 2c6cbc5bc22a1779e9e5168a9459248fab1962c6
+  # and ba4ef23c4de6e13a9160e73e22aa4e01bf1b07ac and rstanarm issue
+  # stan-dev/rstanarm#610, so don't run this test on CRAN:
+  skip_on_cran()
+
   if (exists(".Random.seed", envir = .GlobalEnv)) {
     rng_old <- get(".Random.seed", envir = .GlobalEnv)
   }
@@ -411,7 +417,7 @@ test_that(paste(
     NA
   }
   expect_warning(
-    out_capt <- capture.output({
+    out_capt <- capture.output(
       mfit <- mclogit::mblogit(
         formula = cell ~ treat + age + Karn + prior,
         data = VA,
@@ -419,7 +425,7 @@ test_that(paste(
         model = FALSE,
         y = FALSE
       )
-    }),
+    ),
     warn_expected
   )
   expect_identical(tail(out_capt, 1), "converged")
@@ -653,7 +659,7 @@ test_that(paste(
     NA
   }
   expect_warning(
-    out_capt <- capture.output({
+    out_capt <- capture.output(
       mfit <- mclogit::mblogit(
         formula = cell ~ treat + age + Karn + prior,
         data = VA,
@@ -662,7 +668,7 @@ test_that(paste(
         model = FALSE,
         y = FALSE
       )
-    }),
+    ),
     warn_expected
   )
   expect_identical(tail(out_capt, 1), "converged")
